Class: Mongoid::Association::Referenced::BelongsTo
- Inherits:
-
Object
- Object
- Mongoid::Association::Referenced::BelongsTo
- Includes:
- Buildable, Mongoid::Association::Relatable
- Defined in:
- lib/mongoid/association/referenced/belongs_to.rb,
lib/mongoid/association/referenced/belongs_to/eager.rb,
lib/mongoid/association/referenced/belongs_to/proxy.rb,
lib/mongoid/association/referenced/belongs_to/binding.rb,
lib/mongoid/association/referenced/belongs_to/buildable.rb
Overview
The BelongsTo type association.
Defined Under Namespace
Modules: Buildable Classes: Binding, Eager, Proxy
Constant Summary collapse
- ASSOCIATION_OPTIONS =
The options available for this type of association, in addition to the common ones.
i[ autobuild autosave counter_cache dependent fallback foreign_key index polymorphic primary_key touch optional required scope ].freeze
- VALID_OPTIONS =
The complete list of valid options for this association, including the shared ones.
(ASSOCIATION_OPTIONS + SHARED_OPTIONS).freeze
- FOREIGN_KEY_FIELD_TYPE =
The type of the field holding the foreign key.
Object- FOREIGN_KEY_SUFFIX =
The default foreign key suffix.
'_id'
Constants included from Mongoid::Association::Relatable
Mongoid::Association::Relatable::PRIMARY_KEY_DEFAULT, Mongoid::Association::Relatable::SHARED_OPTIONS
Instance Attribute Summary
Attributes included from Mongoid::Association::Relatable
#name, #options, #owner_class, #parent_inclusions
Instance Method Summary collapse
-
#embedded? ⇒ false
Is this association type embedded?.
-
#foreign_key ⇒ String
Get the foreign key field for saving the association reference.
-
#inverse_type ⇒ String
The name of the field used to store the type of polymorphic association.
-
#nested_builder(attributes, options) ⇒ Association::Nested::One
The nested builder object.
-
#path(document) ⇒ Root
Get the path calculator for the supplied document.
-
#polymorphic? ⇒ true | false
Is this association polymorphic?.
-
#relation ⇒ Association::BelongsTo::Proxy
Get the association proxy class for this association type.
-
#relation_complements ⇒ Array<Mongoid::Association::Relatable>
The list of association complements.
-
#resolver ⇒ nil | Mongoid::ModelResolver
Returns the object responsible for converting polymorphic type references into class objects, and vice versa.
-
#scope ⇒ Proc | Symbol | nil
Get the scope to be applied when querying the association.
-
#setup! ⇒ self
Setup the instance methods, fields, etc.
-
#stores_foreign_key? ⇒ true
Does this association type store the foreign key?.
-
#validation_default ⇒ false
The default for validation the association object.
Methods included from Buildable
Methods included from Mongoid::Association::Relatable
#==, #bindable?, #counter_cache_column_name, #create_relation, #destructive?, #extension, #foreign_key_check, #foreign_key_setter, #get_callbacks, #in_to?, #initialize, #inverse, #inverse_association, #inverse_class, #inverse_class_name, #inverse_setter, #inverse_type_setter, #inverses, #key, #many?, #many_to_many?, #one?, #relation_class, #relation_class_name, #setter, #try_relation_class, #type_setter, #validate?
Methods included from Options
#as, #autobuilding?, #autosave, #cascading_callbacks?, #counter_cached?, #cyclic?, #dependent, #fallback, #fallback?, #forced_nil_inverse?, #indexed?, #inverse_of, #order, #primary_key, #store_as, #touch_field, #touchable?, #type
Methods included from Constrainable
Instance Method Details
#embedded? ⇒ false
Is this association type embedded?
78 79 80 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 78 def false end |
#foreign_key ⇒ String
Get the foreign key field for saving the association reference.
92 93 94 95 96 97 98 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 92 def foreign_key @foreign_key ||= if [:foreign_key] [:foreign_key].to_s else default_foreign_key_field end end |
#inverse_type ⇒ String
The name of the field used to store the type of polymorphic association.
132 133 134 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 132 def inverse_type (@inverse_type ||= "#{name}_type") if polymorphic? end |
#nested_builder(attributes, options) ⇒ Association::Nested::One
The nested builder object.
142 143 144 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 142 def nested_builder(attributes, ) Nested::One.new(self, attributes, ) end |
#path(document) ⇒ Root
Get the path calculator for the supplied document.
154 155 156 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 154 def path(document) Mongoid::Atomic::Paths::Root.new(document) end |
#polymorphic? ⇒ true | false
Is this association polymorphic?
110 111 112 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 110 def polymorphic? @polymorphic ||= !![:polymorphic] end |
#relation ⇒ Association::BelongsTo::Proxy
Get the association proxy class for this association type.
103 104 105 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 103 def relation Proxy end |
#relation_complements ⇒ Array<Mongoid::Association::Relatable>
The list of association complements.
55 56 57 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 55 def relation_complements @relation_complements ||= [ HasMany, HasOne ].freeze end |
#resolver ⇒ nil | Mongoid::ModelResolver
Returns the object responsible for converting polymorphic type references into
class objects, and vice versa. This is obtained via the :polymorphic option
that was given when the association was defined.
See Mongoid::ModelResolver.resolver for how the :polymorphic option is
interpreted here.
125 126 127 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 125 def resolver @resolver ||= Mongoid::ModelResolver.resolver([:polymorphic]) end |
#scope ⇒ Proc | Symbol | nil
Get the scope to be applied when querying the association.
161 162 163 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 161 def scope [:scope] end |
#setup! ⇒ self
Setup the instance methods, fields, etc. on the association owning class.
62 63 64 65 66 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 62 def setup! setup_instance_methods! @owner_class.aliased_fields[name.to_s] = foreign_key self end |
#stores_foreign_key? ⇒ true
Does this association type store the foreign key?
71 72 73 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 71 def stores_foreign_key? true end |
#validation_default ⇒ false
The default for validation the association object.
85 86 87 |
# File 'lib/mongoid/association/referenced/belongs_to.rb', line 85 def validation_default false end |