Module: Mongoid::Association::Relatable

Overview

This module provides behaviors shared between Association types.

Since:

  • 7.0

Constant Summary collapse

SHARED_OPTIONS =

The options shared between all association types.

Since:

  • 7.0

[
  :class_name,
  :inverse_of,
  :validate,
  :extend
].freeze
PRIMARY_KEY_DEFAULT =

The primary key default.

Since:

  • 7.0

'_id'.freeze

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Options

#as, #autobuilding?, #autosave, #cascading_callbacks?, #counter_cached?, #cyclic?, #dependent, #forced_nil_inverse?, #indexed?, #inverse_of, #order, #polymorphic?, #primary_key, #store_as, #touch_field, #type

Methods included from Constrainable

#convert_to_foreign_key

Instance Attribute Details

#nameSymbol (readonly)

The name of the association.

Since:

  • 7.0



41
42
43
# File 'lib/mongoid/association/relatable.rb', line 41

def name
  @name
end

#optionsHash (readonly)

The options on this association.

Since:

  • 7.0



48
49
50
# File 'lib/mongoid/association/relatable.rb', line 48

def options
  @options
end

Instance Method Details

#==(other) ⇒ Object

Compare this association to another.

Since:

  • 7.0



76
77
78
79
80
81
# File 'lib/mongoid/association/relatable.rb', line 76

def ==(other)
  relation_class_name == other.relation_class_name &&
    inverse_class_name == other.inverse_class_name &&
      name == other.name &&
        options == other.options
end

#bindable?(doc) ⇒ true, false

Whether trying to bind an object using this association should raise an error.

Since:

  • 7.0



113
# File 'lib/mongoid/association/relatable.rb', line 113

def bindable?(doc); false; end

#counter_cache_column_nameString

Get the counter cache column name.

Since:

  • 7.0



322
323
324
325
326
# File 'lib/mongoid/association/relatable.rb', line 322

def counter_cache_column_name
  @counter_cache_column_name ||= (@options[:counter_cache].is_a?(String) ||
      @options[:counter_cache].is_a?(Symbol)) ?
      @options[:counter_cache] : "#{inverse || inverse_class_name.demodulize.underscore.pluralize}_count"
end

#create_relation(owner, target) ⇒ Proxy

Create an association proxy object using the owner and target.

Since:

  • 7.0



304
305
306
# File 'lib/mongoid/association/relatable.rb', line 304

def create_relation(owner, target)
  relation.new(owner, target, self)
end

#destructive?true, false

Whether the dependent method is destructive.

Since:

  • 7.0



313
314
315
# File 'lib/mongoid/association/relatable.rb', line 313

def destructive?
  @destructive ||= !!(dependent && (dependent == :delete_all || dependent == :destroy))
end

#extensionModule

Get the extension.

Since:

  • 7.0



333
334
335
# File 'lib/mongoid/association/relatable.rb', line 333

def extension
  @extension ||= @options[:extend]
end

#foreign_key_checkString

Get the name of the method to check if the foreign key has changed.

Examples:

Get the foreign key check method.

association.foreign_key_check

Since:

  • 7.0



291
292
293
# File 'lib/mongoid/association/relatable.rb', line 291

def foreign_key_check
  @foreign_key_check ||= "#{foreign_key}_changed?" if (stores_foreign_key? && foreign_key)
end

#foreign_key_setterString

The name of the foreign key setter method.

Since:

  • 7.0



255
256
257
258
259
# File 'lib/mongoid/association/relatable.rb', line 255

def foreign_key_setter
  # note: You can't check if this association stores foreign key
  # See HasOne and HasMany binding, they referenced foreign_key_setter
  @foreign_key_setter ||= "#{foreign_key}=" if foreign_key
end

#get_callbacks(callback_type) ⇒ Array<Proc, Symbol>

Get the callbacks for a given type.

Since:

  • 7.0



91
92
93
# File 'lib/mongoid/association/relatable.rb', line 91

def get_callbacks(callback_type)
  Array(options[callback_type])
end

#initialize(_class, name, opts = {}, &block) ⇒ Object

Initialize the Association.

Since:

  • 7.0



58
59
60
61
62
63
64
65
66
67
68
69
# File 'lib/mongoid/association/relatable.rb', line 58

def initialize(_class, name, opts = {}, &block)
  @owner_class = _class
  @name = name
  @options = opts
  @extension = nil

  @module_path = _class.name ? _class.name.split('::')[0..-2].join('::') : ''
  @module_path << '::' unless @module_path.empty?

  create_extension!(&block)
  validate!
end

#inverse(other = nil) ⇒ Symbol

Get the inverse name.

Since:

  • 7.0



342
343
344
345
# File 'lib/mongoid/association/relatable.rb', line 342

def inverse(other = nil)
  candidates = inverses(other)
  candidates.detect { |c| c } if candidates
end

#inverse_association(other = nil) ⇒ Association

Get the inverse’s association metadata.

Since:

  • 7.0



140
141
142
# File 'lib/mongoid/association/relatable.rb', line 140

def inverse_association(other = nil)
  (other || relation_class).relations[inverse(other)]
end

#inverse_classString Also known as: inverse_klass

The class of the object owning this association.

Since:

  • 7.0



217
218
219
# File 'lib/mongoid/association/relatable.rb', line 217

def inverse_class
  @owner_class
end

#inverse_class_nameString

The class name of the object owning this association.

Since:

  • 7.0



208
209
210
# File 'lib/mongoid/association/relatable.rb', line 208

def inverse_class_name
  @inverse_class_name ||= @owner_class.name
end

#inverse_setter(other = nil) ⇒ String

The name of the inverse setter method.

Since:

  • 7.0



246
247
248
# File 'lib/mongoid/association/relatable.rb', line 246

def inverse_setter(other = nil)
  @inverse_setter ||= "#{inverses(other).first}=" unless inverses(other).blank?
end

#inverse_typenil

Get the inverse type.

Since:

  • 7.0



149
# File 'lib/mongoid/association/relatable.rb', line 149

def inverse_type; end

#inverse_type_setterString

Gets the setter for the field that sets the type of document on a polymorphic association.

Examples:

Get the inverse type setter.

association.inverse_type_setter

Since:

  • 7.0



279
280
281
# File 'lib/mongoid/association/relatable.rb', line 279

def inverse_type_setter
  @inverse_type_setter ||= inverse_type.__setter__
end

#inverses(other = nil) ⇒ Array<Symbol>

Get the inverse names.

Since:

  • 7.0



123
124
125
126
127
128
129
130
# File 'lib/mongoid/association/relatable.rb', line 123

def inverses(other = nil)
  return [ inverse_of ] if inverse_of
  if polymorphic?
    polymorphic_inverses(other)
  else
    determine_inverses(other)
  end
end

#keySymbol, String

The foreign key field if this association stores a foreign key. Otherwise, the primary key.

Since:

  • 7.0



228
229
230
# File 'lib/mongoid/association/relatable.rb', line 228

def key
  stores_foreign_key? ? foreign_key : primary_key
end

#path(document) ⇒ Mongoid::Atomic::Paths::Root

The atomic path for this association.

Since:

  • 7.0



266
267
268
# File 'lib/mongoid/association/relatable.rb', line 266

def path(document)
  relation.path(document)
end

#relation_classString Also known as: klass

The class of the association object(s).

This method returns the class instance corresponding to relation_class_name, resolved relative to the host document class.

If the class does not exist, this method raises NameError. This can happen because the target class has not yet been defined. Note that polymorphic associations generally do not have a well defined target class because the target class can change from one object to another, and calling this method on a polymorphic association will generally fail with a NameError or produce misleading results (if a class does happen to be defined with the same name as the association name).

Since:

  • 7.0



195
196
197
198
199
200
# File 'lib/mongoid/association/relatable.rb', line 195

def relation_class
  @klass ||= begin
    cls_name = @options[:class_name] || ActiveSupport::Inflector.classify(name)
    resolve_name(inverse_class, cls_name)
  end
end

#relation_class_nameString Also known as: class_name

Note:

The return value of this method should not be used to determine whether two associations have the same target class, because the return value is not always a fully qualified class name. To compare classes, retrieve the class instance of the association target using the relation_class method.

The class name, possibly unqualified or

prefixed, of the association

object(s).

This method returns the class name as it is used in the association definition. If :class_name option is given in the association, the exact value of that option is returned here. If :class_name option is not given, the name of the class is calculated from association name but is not resolved to the actual class.

The class name returned by this method may not correspond to a defined class, either because the corresponding class has not been loaded yet, or because the association references a non-existent class altogether. To obtain the association class, use relation_class method.

Since:

  • 7.0



174
175
176
# File 'lib/mongoid/association/relatable.rb', line 174

def relation_class_name
  @class_name ||= @options[:class_name] || ActiveSupport::Inflector.classify(name)
end

#setterString

The name of the setter on this object for assigning an associated object.

Since:

  • 7.0



237
238
239
# File 'lib/mongoid/association/relatable.rb', line 237

def setter
  @setter ||= "#{name}="
end

#type_setterString

Note:

Only relevant for polymorphic associations that take the :as option.

Get the type setter.

Since:

  • 7.0



101
102
103
# File 'lib/mongoid/association/relatable.rb', line 101

def type_setter
  @type_setter ||= type.__setter__
end

#validate?true, false

Whether the associated object(s) should be validated.

Since:

  • 7.0



353
354
355
356
357
358
359
# File 'lib/mongoid/association/relatable.rb', line 353

def validate?
  @validate ||= if @options[:validate].nil?
                  validation_default
                else
                  !!@options[:validate]
                end
end